#[repr(C)]pub struct BasicColorModifiersPod {
pub rgb_or_hsv: Vec3,
pub alpha: f32,
pub contrast: f32,
pub exposure: f32,
pub gamma: f32,
pub _padding: u32,
}Expand description
The POD representation of the basic color modifiers buffer.
RGB Override or HSV Modifications
If any value of rgb_or_hsv is negative, then it is used to override the RGB color,
otherwise it is used to apply HSV modifications.
For example, to override the color to red, use rgb_or_hsv = Vec3::new(-1.0, 0.0, 0.0),
to apply a hue shift of 180 degrees, use rgb_or_hsv = Vec3::new(0.5, 1.0, 1.0).
Fields
rgb_or_hsv: Vec3If any value of this vector is negative, then it is used to override the RGB color, otherwise it is used to apply HSV modifications.
alpha: f32contrast: f32exposure: f32gamma: f32_padding: u32Implementations
Sourceimpl BasicColorModifiersPod
impl BasicColorModifiersPod