BasicColorRgbOverrideOrHsvModifiersPod

Struct BasicColorRgbOverrideOrHsvModifiersPod 

#[repr(transparent)]
pub struct BasicColorRgbOverrideOrHsvModifiersPod(pub Vec3);
The POD representation of the basic color RGB override or HSV modifiers.

If any value of this vector is negative (including negative zero), then it is used to override the RGB color, otherwise it is used to apply HSV modifications.

Tuple Fields

0: Vec3

Implementations

Source

impl BasicColorRgbOverrideOrHsvModifiersPod

pub fn new_rgb_override(rgb: Vec3) -> Self

Creates a new RGB override pod.

pub const fn new_hsv_modifiers(hsv: Vec3) -> Self

Creates a new HSV modifiers pod.

pub fn is_rgb_override(&self) -> bool

Checks if this pod represents an RGB override.

pub fn is_hsv_modifiers(&self) -> bool

Checks if this pod represents HSV modifiers.

pub fn get_vec3(&self) -> Vec3

Returns the Vec3 value, negating it if this represents an RGB override.