NonDestructiveModifier

Struct NonDestructiveModifier 

pub struct NonDestructiveModifier<G: GaussianPod, M: Modifier<G>> {
    pub modifier: M,
    pub original_gaussians: GaussiansBuffer<G>,
}
A wrapper for any Modifier to not modify the original Gaussians.

This modifier stores a copy of the original Gaussians, whenever it is applied, it first copies original Gaussians to the destination Gaussians buffer, then applies the inner modifier.

Fields

modifier: Moriginal_gaussians: GaussiansBuffer<G>

Implementations

Source

impl<G: GaussianPod, M: Modifier<G>> NonDestructiveModifier<G, M>

pub fn new( device: &Device, queue: &Queue, modifier: M, gaussians: &GaussiansBuffer<G>, ) -> Result<Self, NonDestructiveModifierCreateError>

Create a new non-destructive modifier.

gaussians must have wgpu::BufferUsages::COPY_SRC in its usage.

pub fn try_apply( &self, device: &Device, encoder: &mut CommandEncoder, gaussians: &GaussiansBuffer<G>, model_transform: &ModelTransformBuffer, gaussian_transform: &GaussianTransformBuffer, ) -> Result<(), GaussiansBufferUpdateError>

Try apply the modifier.

Returns Err if the original Gaussians count does not match the target Gaussians count.

pub fn try_apply_with( &self, encoder: &mut CommandEncoder, gaussians: &GaussiansBuffer<G>, f: impl FnOnce(&mut CommandEncoder, &M, &GaussiansBuffer<G>), ) -> Result<(), GaussiansBufferUpdateError>

Apply the modifier with a function.

This is convenient when you are not using the Modifier::apply function, and instead want to use other functions to apply the modifier.