pub struct SelectionOpBuffer(/* private fields */);Expand description
The selection operation uniform buffer for storing selection operations.
This buffer holds a single u32 value representing the index of the selection operation to be
performed, this includes both primitive operations and custom operations, the value is obtained
from SelectionExpr::as_u32, see its documentation for more
details.
Implementations