#[repr(transparent)]pub struct BasicColorRgbOverrideOrHsvModifiersPod(pub Vec3);Expand description
The POD representation of the basic color RGB override or HSV modifiers.
If any value of this vector is negative (including negative zero), then it is used to override the RGB color, otherwise it is used to apply HSV modifications.
Tuple Fields
0: Vec3Implementations
Sourceimpl BasicColorRgbOverrideOrHsvModifiersPod
impl BasicColorRgbOverrideOrHsvModifiersPod
Sourcepub fn new_rgb_override(rgb: Vec3) -> Self
pub fn new_rgb_override(rgb: Vec3) -> Self
Creates a new RGB override pod.
Sourcepub const fn new_hsv_modifiers(hsv: Vec3) -> Self
pub const fn new_hsv_modifiers(hsv: Vec3) -> Self
Creates a new HSV modifiers pod.
Sourcepub fn is_rgb_override(&self) -> bool
pub fn is_rgb_override(&self) -> bool
Checks if this pod represents an RGB override.
Sourcepub fn is_hsv_modifiers(&self) -> bool
pub fn is_hsv_modifiers(&self) -> bool
Checks if this pod represents HSV modifiers.