pub struct SelectionBundle<G: GaussianPod> {
pub bundles: Vec<ComputeBundle<()>>,
/* private fields */
}Expand description
A collection of specialized ComputeBundle for selection operations.
Custom Operations
All ComputeBundles supplied to this bundle as a SelectionExpr::Unary,
SelectionExpr::Binary, or SelectionExpr::Selection custom operation must have the same
bind group 0 as the SelectionBundle::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR. They must also
not have the bind group itself, as it will be supplied automatically during evaluation.
Note that SelectionExpr::Unary will also get the source selection buffer, but it will be
empty (all zeros), you should operate on the destination selection buffer only.
It is recommended to use ComputeBundleBuilder to create the custom operation bundles,
and build them using ComputeBundleBuilder::build_without_bind_groups.
// Create an editor that holds the buffers for the Gaussians
let editor = Editor::new(
&device,
&vec![core::Gaussian {
rot: Quat::IDENTITY,
pos: Vec3::ZERO,
color: U8Vec4::ZERO,
sh: [Vec3::ZERO; 15],
scale: Vec3::ONE,
}],
);
// Create the selection custom operation compute bundle
let my_selection_custom_op_bundle = core::ComputeBundleBuilder::new()
.label("My Selection")
.bind_group_layouts([
&SelectionBundle::<GaussianPod>::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR,
&MY_CUSTOM_BIND_GROUP_LAYOUT_DESCRIPTOR,
])
.resolver({
let mut resolver =
wesl::StandardResolver::new("path/to/my/folder/containing/wesl");
// Required for using core buffer structs.
resolver.add_package(&core::shader::PACKAGE);
// Optionally add this for some utility functions.
resolver.add_package(&shader::PACKAGE);
resolver
})
.main_shader("package::my_wesl_filename".parse().unwrap())
.entry_point("main")
.wesl_compile_options(wesl::CompileOptions {
// Required for enabling the correct features for core struct.
features: GaussianPod::wesl_features(),
..Default::default()
})
.build_without_bind_groups(&device)
.map_err(|e| log::error!("{e}"))
.expect("my selection custom op bundle");
// Create the selection bundle
let selection_bundle =
SelectionBundle::<GaussianPod>::new(&device, vec![my_selection_custom_op_bundle]);
// Create the bind group for your custom operation
let my_selection_custom_op_bind_group = selection_bundle.bundles[0]
.create_bind_group(
&device,
1, // Index 0 is always the Gaussians buffer
[my_buffer.buffer().as_entire_binding()],
)
.unwrap();
// Create the selection expression
let selection_expr = SelectionExpr::selection(
0, // The bundle index for your custom operation in the selection bundle
vec![my_selection_custom_op_bind_group],
)
.union(
// Combine with other selection expressions using different functions
// Here is an existing selection buffer for example
SelectionExpr::Buffer(my_existing_selection_buffer),
);
// Create a selection buffer for the result
let dest_selection_buffer =
SelectionBuffer::new(&device, editor.gaussians_buffer.len() as u32);
// Evaluate the selection expression
selection_bundle.evaluate(
&device,
&mut encoder,
&selection_expr,
&dest_selection_buffer,
&editor.model_transform_buffer,
&editor.gaussian_transform_buffer,
&editor.gaussians_buffer,
);Shader Format
You may copy and paste the following shader bindings for
SelectionBundle::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR into your custom selection operation
shader to ensure that the bindings are correct, then add your own bindings after that.
import wgpu_3dgs_core::{
gaussian::Gaussian,
gaussian_transform::GaussianTransform,
model_transform::{model_to_world, ModelTransform},
};
@group(0) @binding(0)
var<uniform> op: u32;
@group(0) @binding(1)
var<storage, read> source: array<u32>;
@group(0) @binding(2)
var<storage, read_write> dest: array<atomic<u32>>;
@group(0) @binding(3)
var<uniform> model_transform: ModelTransform;
@group(0) @binding(4)
var<uniform> gaussian_transform: GaussianTransform;
@group(0) @binding(5)
var<storage, read> gaussians: array<Gaussian>;
// Your custom bindings here...
override workgroup_size: u32;
@compute @workgroup_size(workgroup_size)
fn main(@builtin(global_invocation_id) id: vec3<u32>) {
let index = id.x;
if index >= arrayLength(&gaussians) {
return;
}
let gaussian = gaussians[index];
let world_pos = model_to_world(model_transform, gaussian.position);
// Your custom selection operation code here...
let word_index = index / 32u;
let bit_index = index % 32u;
let bit_mask = 1u << bit_index;
if /* Condition for selecting the Gaussian */ {
atomicOr(&dest[word_index], bit_mask);
} else {
atomicAnd(&dest[word_index], ~bit_mask);
}
}Fields
bundles: Vec<ComputeBundle<()>>The compute bundles for selection custom operations.
Implementations
Sourceimpl<G: GaussianPod> SelectionBundle<G>
impl<G: GaussianPod> SelectionBundle<G>
Sourcepub const GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR: BindGroupLayoutDescriptor<'static>
pub const GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR: BindGroupLayoutDescriptor<'static>
The Gaussians bind group layout descriptors.
This bind group layout takes the following buffers:
Sourcepub const SPHERE_BIND_GROUP_LAYOUT_DESCRIPTOR: BindGroupLayoutDescriptor<'static>
pub const SPHERE_BIND_GROUP_LAYOUT_DESCRIPTOR: BindGroupLayoutDescriptor<'static>
The sphere selection bind group layout descriptor.
This bind group layout takes the following buffers:
Sourcepub const BOX_BIND_GROUP_LAYOUT_DESCRIPTOR: BindGroupLayoutDescriptor<'static>
pub const BOX_BIND_GROUP_LAYOUT_DESCRIPTOR: BindGroupLayoutDescriptor<'static>
The box selection bind group layout descriptor.
This bind group layout takes the following buffers:
Sourcepub fn new(device: &Device, bundles: Vec<ComputeBundle<()>>) -> Self
pub fn new(device: &Device, bundles: Vec<ComputeBundle<()>>) -> Self
Create a new selection bundle.
bundles are used for SelectionExpr::Unary, SelectionExpr::Binary, or
SelectionExpr::Selection as custom operations, they must have the same bind group 0 as
the SelectionBundle::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR, see documentation of
SelectionBundle for more details.
Examples found in repository?
98async fn main() {
99 env_logger::Builder::from_env(env_logger::Env::default().default_filter_or("info")).init();
100
101 let args = Args::parse();
102 let model_path = &args.model;
103 let pos = Vec3::from_slice(&args.pos);
104 let rot = Quat::from_slice(&args.rot);
105 let scale = Vec3::from_slice(&args.scale);
106 let shape = match args.shape {
107 Shape::Sphere => gs::SelectionBundle::<GaussianPod>::create_sphere_bundle,
108 Shape::Box => gs::SelectionBundle::<GaussianPod>::create_box_bundle,
109 };
110 let repeat = args.repeat;
111 let offset = Vec3::from_slice(&args.offset);
112
113 log::debug!("Creating wgpu instance");
114 let instance = wgpu::Instance::new(&wgpu::InstanceDescriptor::default());
115
116 log::debug!("Requesting adapter");
117 let adapter = instance
118 .request_adapter(&wgpu::RequestAdapterOptions::default())
119 .await
120 .expect("adapter");
121
122 log::debug!("Requesting device");
123 let (device, queue) = adapter
124 .request_device(&wgpu::DeviceDescriptor {
125 label: Some("Device"),
126 required_limits: adapter.limits(),
127 ..Default::default()
128 })
129 .await
130 .expect("device");
131
132 log::debug!("Creating gaussians");
133 let gaussians = [
134 gs::core::GaussiansSource::Ply,
135 gs::core::GaussiansSource::Spz,
136 ]
137 .into_iter()
138 .find_map(|source| gs::core::Gaussians::read_from_file(model_path, source).ok())
139 .expect("gaussians");
140
141 log::debug!("Creating gaussians buffer");
142 let gaussians_buffer = gs::core::GaussiansBuffer::<GaussianPod>::new(&device, &gaussians);
143
144 log::debug!("Creating model transform buffer");
145 let model_transform = gs::core::ModelTransformBuffer::new(&device);
146
147 log::debug!("Creating Gaussian transform buffer");
148 let gaussian_transform = gs::core::GaussianTransformBuffer::new(&device);
149
150 log::debug!("Creating shape selection compute bundle");
151 let shape_selection = shape(&device);
152
153 log::debug!("Creating selection bundle");
154 let selection_bundle = gs::SelectionBundle::<GaussianPod>::new(&device, vec![shape_selection]);
155
156 log::debug!("Creating shape selection buffers");
157 let shape_selection_buffers = (0..repeat)
158 .map(|i| {
159 let offset_pos = pos + offset * i as f32;
160 let buffer = gs::InvTransformBuffer::new(&device);
161 buffer.update_with_scale_rot_pos(&queue, scale, rot, offset_pos);
162 buffer
163 })
164 .collect::<Vec<_>>();
165
166 log::debug!("Creating shape selection bind groups");
167 let shape_selection_bind_groups = shape_selection_buffers
168 .iter()
169 .map(|buffer| {
170 selection_bundle.bundles[0]
171 .create_bind_group(
172 &device,
173 // index 0 is the Gaussians buffer, so we use 1,
174 // see docs of create_sphere_bundle or create_box_bundle
175 1,
176 [buffer.buffer().as_entire_binding()],
177 )
178 .expect("bind group")
179 })
180 .collect::<Vec<_>>();
181
182 log::debug!("Creating selection expression");
183 let selection_expr = shape_selection_bind_groups.into_iter().fold(
184 gs::SelectionExpr::Identity,
185 |acc, bind_group| {
186 acc.union(gs::SelectionExpr::selection(
187 0, // the 0 here is the bundle index in the selection bundle
188 vec![bind_group],
189 ))
190 },
191 );
192
193 log::debug!("Creating destination buffer");
194 let dest = gs::SelectionBuffer::new(&device, gaussians_buffer.len() as u32);
195
196 log::info!("Starting selection process");
197 let time = std::time::Instant::now();
198
199 log::debug!("Selecting Gaussians");
200 let mut encoder = device.create_command_encoder(&wgpu::CommandEncoderDescriptor {
201 label: Some("Selection Encoder"),
202 });
203
204 selection_bundle.evaluate(
205 &device,
206 &mut encoder,
207 &selection_expr,
208 &dest,
209 &model_transform,
210 &gaussian_transform,
211 &gaussians_buffer,
212 );
213
214 queue.submit(Some(encoder.finish()));
215
216 device
217 .poll(wgpu::PollType::wait_indefinitely())
218 .expect("poll");
219
220 log::info!("Editing process completed in {:?}", time.elapsed());
221
222 log::debug!("Filtering Gaussians");
223 let selected_gaussians = dest
224 .download::<u32>(&device, &queue)
225 .await
226 .expect("selected download")
227 .iter()
228 .flat_map(|group| {
229 std::iter::repeat_n(group, 32)
230 .enumerate()
231 .map(|(i, g)| g & (1 << i) != 0)
232 })
233 .zip(gaussians.iter_gaussian())
234 .filter(|(selected, _)| *selected)
235 .map(|(_, g)| g)
236 .collect::<Vec<_>>();
237
238 let selected_gaussians = match &args.output[args.output.len().saturating_sub(4)..] {
239 ".ply" => gs::core::Gaussians::Ply(gs::core::PlyGaussians::from_iter(
240 selected_gaussians.into_iter(),
241 )),
242 ".spz" => {
243 gs::core::Gaussians::Spz(
244 gs::core::SpzGaussians::from_gaussians_with_options(
245 selected_gaussians,
246 &gs::core::SpzGaussiansFromGaussianSliceOptions {
247 version: 2, // Version 2 is more widely supported as of now
248 ..Default::default()
249 },
250 )
251 .expect("SpzGaussians from gaussians"),
252 )
253 }
254 _ => panic!("Unsupported output file extension, expected .ply or .spz"),
255 };
256
257 log::debug!("Writing modified Gaussians to output file");
258 selected_gaussians
259 .write_to_file(&args.output)
260 .expect("write modified Gaussians to output file");
261
262 log::info!("Filtered Gaussians written to {}", args.output);
263}Sourcepub fn gaussians_bind_group_layout(&self) -> &BindGroupLayout
pub fn gaussians_bind_group_layout(&self) -> &BindGroupLayout
Get the Gaussians bind group layout.
Sourcepub fn evaluate(
&self,
device: &Device,
encoder: &mut CommandEncoder,
expr: &SelectionExpr,
dest: &SelectionBuffer,
model_transform: &ModelTransformBuffer,
gaussian_transform: &GaussianTransformBuffer,
gaussians: &GaussiansBuffer<G>,
)
pub fn evaluate( &self, device: &Device, encoder: &mut CommandEncoder, expr: &SelectionExpr, dest: &SelectionBuffer, model_transform: &ModelTransformBuffer, gaussian_transform: &GaussianTransformBuffer, gaussians: &GaussiansBuffer<G>, )
Evaluate and apply the selection expression.
Examples found in repository?
98async fn main() {
99 env_logger::Builder::from_env(env_logger::Env::default().default_filter_or("info")).init();
100
101 let args = Args::parse();
102 let model_path = &args.model;
103 let pos = Vec3::from_slice(&args.pos);
104 let rot = Quat::from_slice(&args.rot);
105 let scale = Vec3::from_slice(&args.scale);
106 let shape = match args.shape {
107 Shape::Sphere => gs::SelectionBundle::<GaussianPod>::create_sphere_bundle,
108 Shape::Box => gs::SelectionBundle::<GaussianPod>::create_box_bundle,
109 };
110 let repeat = args.repeat;
111 let offset = Vec3::from_slice(&args.offset);
112
113 log::debug!("Creating wgpu instance");
114 let instance = wgpu::Instance::new(&wgpu::InstanceDescriptor::default());
115
116 log::debug!("Requesting adapter");
117 let adapter = instance
118 .request_adapter(&wgpu::RequestAdapterOptions::default())
119 .await
120 .expect("adapter");
121
122 log::debug!("Requesting device");
123 let (device, queue) = adapter
124 .request_device(&wgpu::DeviceDescriptor {
125 label: Some("Device"),
126 required_limits: adapter.limits(),
127 ..Default::default()
128 })
129 .await
130 .expect("device");
131
132 log::debug!("Creating gaussians");
133 let gaussians = [
134 gs::core::GaussiansSource::Ply,
135 gs::core::GaussiansSource::Spz,
136 ]
137 .into_iter()
138 .find_map(|source| gs::core::Gaussians::read_from_file(model_path, source).ok())
139 .expect("gaussians");
140
141 log::debug!("Creating gaussians buffer");
142 let gaussians_buffer = gs::core::GaussiansBuffer::<GaussianPod>::new(&device, &gaussians);
143
144 log::debug!("Creating model transform buffer");
145 let model_transform = gs::core::ModelTransformBuffer::new(&device);
146
147 log::debug!("Creating Gaussian transform buffer");
148 let gaussian_transform = gs::core::GaussianTransformBuffer::new(&device);
149
150 log::debug!("Creating shape selection compute bundle");
151 let shape_selection = shape(&device);
152
153 log::debug!("Creating selection bundle");
154 let selection_bundle = gs::SelectionBundle::<GaussianPod>::new(&device, vec![shape_selection]);
155
156 log::debug!("Creating shape selection buffers");
157 let shape_selection_buffers = (0..repeat)
158 .map(|i| {
159 let offset_pos = pos + offset * i as f32;
160 let buffer = gs::InvTransformBuffer::new(&device);
161 buffer.update_with_scale_rot_pos(&queue, scale, rot, offset_pos);
162 buffer
163 })
164 .collect::<Vec<_>>();
165
166 log::debug!("Creating shape selection bind groups");
167 let shape_selection_bind_groups = shape_selection_buffers
168 .iter()
169 .map(|buffer| {
170 selection_bundle.bundles[0]
171 .create_bind_group(
172 &device,
173 // index 0 is the Gaussians buffer, so we use 1,
174 // see docs of create_sphere_bundle or create_box_bundle
175 1,
176 [buffer.buffer().as_entire_binding()],
177 )
178 .expect("bind group")
179 })
180 .collect::<Vec<_>>();
181
182 log::debug!("Creating selection expression");
183 let selection_expr = shape_selection_bind_groups.into_iter().fold(
184 gs::SelectionExpr::Identity,
185 |acc, bind_group| {
186 acc.union(gs::SelectionExpr::selection(
187 0, // the 0 here is the bundle index in the selection bundle
188 vec![bind_group],
189 ))
190 },
191 );
192
193 log::debug!("Creating destination buffer");
194 let dest = gs::SelectionBuffer::new(&device, gaussians_buffer.len() as u32);
195
196 log::info!("Starting selection process");
197 let time = std::time::Instant::now();
198
199 log::debug!("Selecting Gaussians");
200 let mut encoder = device.create_command_encoder(&wgpu::CommandEncoderDescriptor {
201 label: Some("Selection Encoder"),
202 });
203
204 selection_bundle.evaluate(
205 &device,
206 &mut encoder,
207 &selection_expr,
208 &dest,
209 &model_transform,
210 &gaussian_transform,
211 &gaussians_buffer,
212 );
213
214 queue.submit(Some(encoder.finish()));
215
216 device
217 .poll(wgpu::PollType::wait_indefinitely())
218 .expect("poll");
219
220 log::info!("Editing process completed in {:?}", time.elapsed());
221
222 log::debug!("Filtering Gaussians");
223 let selected_gaussians = dest
224 .download::<u32>(&device, &queue)
225 .await
226 .expect("selected download")
227 .iter()
228 .flat_map(|group| {
229 std::iter::repeat_n(group, 32)
230 .enumerate()
231 .map(|(i, g)| g & (1 << i) != 0)
232 })
233 .zip(gaussians.iter_gaussian())
234 .filter(|(selected, _)| *selected)
235 .map(|(_, g)| g)
236 .collect::<Vec<_>>();
237
238 let selected_gaussians = match &args.output[args.output.len().saturating_sub(4)..] {
239 ".ply" => gs::core::Gaussians::Ply(gs::core::PlyGaussians::from_iter(
240 selected_gaussians.into_iter(),
241 )),
242 ".spz" => {
243 gs::core::Gaussians::Spz(
244 gs::core::SpzGaussians::from_gaussians_with_options(
245 selected_gaussians,
246 &gs::core::SpzGaussiansFromGaussianSliceOptions {
247 version: 2, // Version 2 is more widely supported as of now
248 ..Default::default()
249 },
250 )
251 .expect("SpzGaussians from gaussians"),
252 )
253 }
254 _ => panic!("Unsupported output file extension, expected .ply or .spz"),
255 };
256
257 log::debug!("Writing modified Gaussians to output file");
258 selected_gaussians
259 .write_to_file(&args.output)
260 .expect("write modified Gaussians to output file");
261
262 log::info!("Filtered Gaussians written to {}", args.output);
263}Sourcepub fn create_primitive_bundle(device: &Device) -> ComputeBundle<()>
pub fn create_primitive_bundle(device: &Device) -> ComputeBundle<()>
Create the selection primitive operation ComputeBundle.
- Bind group 0 is
SelectionBundle::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR.
You usually do not need to use this method, it is used internally for creating the primitive operation bundle for evaluation.
Sourcepub fn create_sphere_bundle(device: &Device) -> ComputeBundle<()>
pub fn create_sphere_bundle(device: &Device) -> ComputeBundle<()>
Create a sphere selection custom operation.
- Bind group 0 is
SelectionBundle::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR. - Bind group 1 is
SelectionBundle::SPHERE_BIND_GROUP_LAYOUT_DESCRIPTOR.
Sourcepub fn create_box_bundle(device: &Device) -> ComputeBundle<()>
pub fn create_box_bundle(device: &Device) -> ComputeBundle<()>
Create a box selection custom operation.
- Bind group 0 is
SelectionBundle::GAUSSIANS_BIND_GROUP_LAYOUT_DESCRIPTOR. - Bind group 1 is
SelectionBundle::BOX_BIND_GROUP_LAYOUT_DESCRIPTOR.